package eu.digitalconcepts.dcgw.api.telegram;

import java.io.IOException;

import com.fasterxml.jackson.core.JsonGenerationException;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import eu.digitalconcepts.dcgw.api.telegram.FunctionValue.FunctionValueDeserializer;
import eu.digitalconcepts.dcgw.api.telegram.FunctionValue.FunctionValueSerializer;


/**
 * Model of a function value which can be a numeric value or a key value.
 * 
 * 
 * @author  Digital Concepts GmbH, T. Hennerich
 */
@JsonSerialize(  using = FunctionValueSerializer.class)
@JsonDeserialize(using = FunctionValueDeserializer.class)
public class FunctionValue
{
	// ===================================================================================================================================
	// Field Variables
	// ===================================================================================================================================

	/**
	 * Value as string.
	 */
	private String value;
	

	// ===================================================================================================================================
	// Getter / Setter
	// ===================================================================================================================================

	public String getValueAsString()
	{
		return value;
	}

	public Double getValueAsDouble()
	{
		try
		{
			return Double.valueOf(value);
		}
		catch(Exception e)
		{
		}
		
		return null;
	}
	
	public void setValue(String pValue)
	{
		value = pValue;
	}
	
	public void setValue(Double pValue)
	{
		value = pValue.toString();
	}
	
	
	// ===================================================================================================================================
	// Info
	// ===================================================================================================================================
	
	public boolean isStringValue()
	{
		return !isDoubleValue();
	}
	
	public boolean isDoubleValue()
	{
		if(value==null)
			return false;
		
		for(int i=0; i<value.length(); i++)
		{
			char digit = value.charAt(i);
			
			if((digit<'0' || digit>'9') && digit!='.')
				return false;
		}
		
		return true;
	}
	
	@Override
	public String toString()
	{
		return value;
	}

	
	// ===================================================================================================================================
	// JSON Conversion
	// ===================================================================================================================================

	/**
	 * Serializes the value as string or number.
	 */
	public static class FunctionValueSerializer extends JsonSerializer<FunctionValue>
	{
		@Override
		public void serialize(FunctionValue pValue, JsonGenerator pGenerator, SerializerProvider pProvider) throws IOException, JsonGenerationException 
	    {
	        if(pValue==null || pValue.value==null) 
	            pGenerator.writeNull();
	        else if(pValue.isStringValue())
	        	pGenerator.writeString(pValue.value);
	        else
	    		pGenerator.writeNumber(pValue.value);
	    }
	}
	
	/**
	 * Deserializes a string or number to a value entity.
	 */
	public static class FunctionValueDeserializer extends JsonDeserializer<FunctionValue>
	{
		@Override
		public FunctionValue deserialize(JsonParser pJsonParser, DeserializationContext pContext) throws IOException, JsonProcessingException
		{
			FunctionValue value = new FunctionValue();
			value.value = pJsonParser.getText();
			
			return value;
		}
	}
}