package eu.digitalconcepts.dcgw.client.util;


/**
 *  Non streaming Base64 encoder/decoder.
 *  
 *  @author  unknown
 */
public class Base64
{
	/**
	 * Decode data and return bytes.
	 */
	public final static byte[] decode(String pString)
	{
		if(pString==null)
			return null;
		
		byte data[] = pString.getBytes();
		
		return decode(data);
	}
	
	/**
	 * Decode data and return bytes. Assumes that the data passed in is ASCII text.
	 */
	public final static byte[] decode(byte[] pData)
	{
		int tail = pData.length;
		
		while (pData[tail - 1] == '=')
			tail--;
		
		byte dest[] = new byte[tail - pData.length / 4];

		// ASCII printable to 0-63 conversion
		for (int i=0; i<pData.length; i++)
		{
			if (pData[i] == '=')
				pData[i] = 0;
			else if (pData[i] == '/')
				pData[i] = 63;
			else if (pData[i] == '+')
				pData[i] = 62;
			else if (pData[i] >= '0' && pData[i] <= '9')
				pData[i] = (byte) (pData[i] - ('0' - 52));
			else if (pData[i] >= 'a' && pData[i] <= 'z')
				pData[i] = (byte) (pData[i] - ('a' - 26));
			else if (pData[i] >= 'A' && pData[i] <= 'Z')
				pData[i] = (byte) (pData[i] - 'A');
		}

		// 4-byte to 3-byte conversion
		int s;
		int d;
		
		for(s=0, d=0; d<dest.length-2; s+=4, d+=3)
		{
			dest[d] = (byte) (((pData[s] << 2) & 255) | ((pData[s + 1] >>> 4) & 3));
			dest[d + 1] = (byte) (((pData[s + 1] << 4) & 255) | ((pData[s + 2] >>> 2) & 017));
			dest[d + 2] = (byte) (((pData[s + 2] << 6) & 255) | (pData[s + 3] & 077));
		}
		
		if (d < dest.length)
			dest[d] = (byte) (((pData[s] << 2) & 255) | ((pData[s + 1] >>> 4) & 3));

		if (++d < dest.length)
			dest[d] = (byte) (((pData[s + 1] << 4) & 255) | ((pData[s + 2] >>> 2) & 017));

		return dest;
	}

	/**
	 * Encode string.
	 */
	public final static String encode(String pString)
	{
		return encode(pString.getBytes());
	}
	
	/**
	 * Encode data and return a String.
	 */
	public final static String encode(byte[] pData)
	{
		if(pData==null)
			return null;
		
		byte data[] = new byte[pData.length + 2];
		System.arraycopy(pData, 0, data, 0, pData.length);
		byte dest[] = new byte[(data.length / 3) * 4];

		// 3-byte to 4-byte conversion
		for(int s=0, d=0; s < pData.length; s+=3, d+=4)
		{
			dest[d] = (byte) ((data[s] >>> 2) & 077);
			dest[d + 1] = (byte) ((data[s + 1] >>> 4) & 017 | (data[s] << 4) & 077);
			dest[d + 2] = (byte) ((data[s + 2] >>> 6) & 003 | (data[s + 1] << 2) & 077);
			dest[d + 3] = (byte) (data[s + 2] & 077);
		}

		// 0-63 to ascii printable conversion
		for (int idx = 0; idx < dest.length; idx++)
		{
			if (dest[idx] < 26)
				dest[idx] = (byte) (dest[idx] + 'A');
			else if (dest[idx] < 52)
				dest[idx] = (byte) (dest[idx] + 'a' - 26);
			else if (dest[idx] < 62)
				dest[idx] = (byte) (dest[idx] + '0' - 52);
			else if (dest[idx] < 63)
				dest[idx] = (byte) '+';
			else
				dest[idx] = (byte) '/';
		}

		// add padding
		for (int idx = dest.length - 1; idx > (pData.length * 4) / 3; idx--)
			dest[idx] = (byte) '=';

		return new String(dest);
	}
}
